/* Copyright (c) 2011 Peter Troshin
 * Copyright (c) 2013 Alexander Sherstnev
 *  
 *  JAva Bioinformatics Analysis Web Services (JABAWS)
 *  @version: 2.5     
 * 
 *  This library is free software; you can redistribute it and/or modify it under the terms of the
 *  Apache License version 2 as published by the Apache Software Foundation
 * 
 *  This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the Apache 
 *  License for more details.
 * 
 *  A copy of the license is in apache_license.txt. It is also available here:
 * @see: http://www.apache.org/licenses/LICENSE-2.0.txt
 * 
 * Any republication or derived work distributed in source code form
 * must include this copyright and license notice.
 */
package compbio.ws.server;

import java.io.File;
import java.util.Arrays;
import java.util.List;

import javax.jws.WebService;

import org.apache.log4j.Logger;

import compbio.data.msa.JABAService;
import compbio.data.msa.SequenceAnnotation;
import compbio.data.sequence.FastaSequence;
import compbio.engine.Configurator;
import compbio.engine.client.ConfiguredExecutable;
import compbio.metadata.ChunkHolder;
import compbio.metadata.JobSubmissionException;
import compbio.metadata.LimitExceededException;
import compbio.metadata.Option;
import compbio.metadata.Preset;
import compbio.metadata.UnsupportedRuntimeException;
import compbio.metadata.WrongParameterException;
import compbio.runner.predictors.Jpred;

@WebService(endpointInterface = "compbio.data.msa.SequenceAnnotation", targetNamespace = JABAService.V2_SERVICE_NAMESPACE, serviceName = "JpredWS")
public class JpredWS extends SequenceAnnotationService<Jpred>
		implements SequenceAnnotation<Jpred> {

	private static Logger log = Logger.getLogger(JpredWS.class);

	public JpredWS() {
		super(new Jpred(), log);
	}

	@Override
	public String analize(List<FastaSequence> sequences)
			throws UnsupportedRuntimeException, LimitExceededException, JobSubmissionException {
		WSUtil.validateJpredInput(sequences);
		ConfiguredExecutable<Jpred> confpred = init(sequences);

		// set default conservation method to fastest - SHENKIN
		// TODO: This violates encapsulation, should be moved to the runners
		// level.
		//confpred.addParameters(Arrays.asList("-m=SHENKIN"));
		return WSUtil.analize(sequences, confpred, log, "JpredWS analize", getLimit(""));
	}

	@Override
	public String customAnalize(List<FastaSequence> sequences, List<Option<Jpred>> options) 
					throws UnsupportedRuntimeException, LimitExceededException, JobSubmissionException, WrongParameterException {
		WSUtil.validateJpredInput(sequences);
		return super.customAnalize(sequences, options);
	}

	@Override
	public String presetAnalize(List<FastaSequence> sequences, Preset<Jpred> preset)
			throws UnsupportedRuntimeException, LimitExceededException, JobSubmissionException, WrongParameterException {
		WSUtil.validateJpredInput(sequences);
		return super.presetAnalize(sequences, preset);
	}

	@Override
	public ChunkHolder pullExecStatistics(String jobId, long position) {
		WSUtil.validateJobId(jobId);
		String file = Configurator.getWorkDirectory(jobId) + File.separator + Jpred.getStatFile();
		return WSUtil.pullFile(file, position);
	}

}
